==> bcmd-0.5.2-py3-none-any.whl: bcmd/tasks/image.py <==
import asyncio
import os
import random
from enum import StrEnum
from pathlib import Path
from typing import Final

import httpx
import typer
from beni import bcolor, bfile, bhttp, block, bpath, btask
from beni.bbyte import BytesReader, BytesWriter
from beni.bfunc import syncCall
from beni.btype import Null, XPath
from PIL import Image

app: Final = btask.newSubApp('图片工具集')


class _OutputType(StrEnum):
    '输出类型'
    normal = '0'
    replace_ = '1'
    crc_replace = '2'


@app.command()
@syncCall
async def convert(
    path: Path = typer.Option(None, '--path', '-p', help='指定目录或具体图片文件，默认当前目录'),
    src_format: str = typer.Option('jpg|jpeg|png', '--src-format', '-s', help='如果path是目录，指定源格式，可以指定多个，默认值：jpg|jpeg|png'),
    dst_format: str = typer.Option('webp', '--dst-format', '-d', help='目标格式，只能是单个'),
    rgb: bool = typer.Option(False, '--rgb', help='转换为RGB格式'),
    quality: int = typer.Option(85, '--quality', '-q', help='图片质量，0-100，默认85'),
    output_type: _OutputType = typer.Option(_OutputType.normal, '--output-type', help='输出类型，0：普通输出，1：删除源文件，2：输出文件使用CRC32命名并删除源文件'),
):
    '图片格式转换'
    path = path or Path(os.getcwd())
    fileList: list[Path] = []
    if path.is_file():
        fileList.append(path)
    elif path.is_dir():
        extNameList = [x for x in src_format.strip().split('|')]
        fileList = [x for x in bpath.listFile(path, True) if x.suffix[1:].lower() in extNameList]
    if not fileList:
        return bcolor.printRed(f'未找到图片文件（{path}）')
    for file in fileList:
        with Image.open(file) as img:
            if rgb:
                img = img.convert('RGB')
            with bpath.useTempFile() as tempFile:
                img.save(tempFile, format=dst_format, quality=quality)
                outputFile = file.with_suffix(f'.{dst_format}')
                if output_type == _OutputType.crc_replace:
                    outputFile = outputFile.with_stem(await bfile.crc(tempFile))
                bpath.copy(tempFile, outputFile)
                if output_type in [_OutputType.replace_, _OutputType.crc_replace]:
                    if outputFile != file:
                        bpath.remove(file)
                bcolor.printGreen(f'{file} -> {outputFile}')


# ------------------------------------------------------------------------------------

@app.command()
@syncCall
async def tiny(
    path: Path = typer.Option(None, '--path', help='指定目录或具体图片文件，默认当前目录'),
    compression: int = typer.Option(75, '--compression', help='如果压缩小于指定数值则使用原来的图片，单位：%，默认75'),
    isKeepOriginal: bool = typer.Option(False, '--keep-original', help='保留原始图片'),
):

    keyList = [
        'MB3QmtvZ8HKRkXcDnxhWCNTXzvx6cNF3',
        '7L7X2CJ35GM1bChSHdT14yZPLx7FlpNk',
        'q8YLcvrXVW2NYcr5mMyzQhsSHF4j7gny',
    ]
    random.shuffle(keyList)
    await block.setLimit(_TinyFile.runTiny, len(keyList))

    # 整理文件列表
    fileList = []
    path = path or Path(os.getcwd())
    if path.is_file():
        fileList.append(path)
    elif path.is_dir():
        fileList = [x for x in bpath.listFile(path, True) if x.suffix[1:].lower() in ['jpg', 'jpeg', 'png']]
    else:
        btask.abort('未找到图片文件', path)
    fileList.sort()

    # 将文件列表整理成 _TinyFile 对象
    fileList = [_TinyFile(x) for x in fileList]
    await asyncio.gather(*[x.updateInfo() for x in fileList])

    # 过滤掉已经处理过的图片
    for i in range(len(fileList)):
        file = fileList[i]
        if file.compression == 0:
            # 未处理过的图片，进行图片的压缩处理
            pass
        elif not file.isTiny and file.compression < compression:
            # 之前测试的压缩率不符合要求，不过现在符合了，进行图片的压缩处理
            pass
        else:
            # 要忽略掉的文件
            bcolor.printYellow(f'{file.file}（{file.compression}% / 忽略）')
            fileList[i] = Null
    fileList = [x for x in fileList if x]

    # 开始压缩
    taskList = [
        x.runTiny(keyList[i % len(keyList)], compression, isKeepOriginal)
        for i, x in enumerate(fileList)
    ]
    await asyncio.gather(*taskList)


# ------------------------------------------------------------------------------------


class _TinyFile:

    _endian: Final = '>'
    _sep = BytesWriter(_endian).writeStr('tiny').writeUint(9527).writeUint(709394).toBytes()

    @property
    def compression(self):
        return self._compression

    @property
    def isTiny(self):
        return self._isTiny

    @property
    def file(self):
        return self._file

    def __init__(self, file: XPath):
        self._file = file
        self._compression: float = 0.0
        self._isTiny: bool = False

    async def updateInfo(self):
        fileBytes = await bfile.readBytes(self._file)
        self._compression = 0.0
        self._isTiny = False
        blockAry = fileBytes.split(self._sep)
        if len(blockAry) > 1:
            info = BytesReader(self._endian, blockAry[1])
            size = info.readUint()
            if size == len(blockAry[0]):
                self._compression = round(info.readFloat(), 2)
                self._isTiny = info.readBool()

    async def _flushInfo(self, compression: float, isTiny: bool):
        self._compression = compression
        self._isTiny = isTiny
        content = await self._getPureContent()
        info = (
            BytesWriter(self._endian)
            .writeUint(len(content))
            .writeFloat(compression)
            .writeBool(isTiny)
            .toBytes()
        )
        content += self._sep + info
        await bfile.writeBytes(self._file, content)

    async def _getPureContent(self):
        content = await bfile.readBytes(self._file)
        content = content.split(self._sep)[0]
        return content

    @block.limit(1)
    async def runTiny(self, key: str, compression: float, isKeepOriginal: bool):
        content = await self._getPureContent()
        async with httpx.AsyncClient() as client:
            response = await client.post(
                'https://api.tinify.com/shrink',
                auth=('api', key),
                content=content,
                timeout=30,
            )
            response.raise_for_status()
            result = response.json()
            outputCompression = round(result['output']['ratio'] * 100, 2)
            if outputCompression < compression:
                # 下载文件
                url = result['output']['url']
                with bpath.useTempFile() as tempFile:
                    await bhttp.download(url, tempFile)
                    await _TinyFile(tempFile)._flushInfo(outputCompression, True)
                    outputFile = bpath.get(self._file)
                    if isKeepOriginal:
                        outputFile = outputFile.with_stem(f'{outputFile.stem}_tiny')
                    bpath.move(tempFile, outputFile, True)
                    bcolor.printGreen(f'{outputFile}（{outputCompression}% / 已压缩）')
            else:
                # 不进行压缩
                await self._flushInfo(outputCompression, False)
                bcolor.printMagenta(f'{self._file} （{outputCompression}% / 不处理）')

==> bcmd-0.5.15-py3-none-any.whl: bcmd/tasks/image.py <==
import asyncio
import os
import random
from enum import StrEnum
from pathlib import Path
from typing import Final

import httpx
import typer
from beni import bcolor, bfile, bhttp, block, bpath, btask
from beni.bbyte import BytesReader, BytesWriter
from beni.bfunc import syncCall
from beni.btype import Null, XPath
from PIL import Image

app: Final = btask.newSubApp('图片工具集')


class _OutputType(StrEnum):
    '输出类型'
    normal = '0'
    replace_ = '1'
    crc_replace = '2'


@app.command()
@syncCall
async def convert(
    path: Path = typer.Option(None, '--path', '-p', help='指定目录或具体图片文件，默认当前目录'),
    src_format: str = typer.Option('jpg|jpeg|png', '--src-format', '-s', help='如果path是目录，指定源格式，可以指定多个，默认值：jpg|jpeg|png'),
    dst_format: str = typer.Option('webp', '--dst-format', '-d', help='目标格式，只能是单个'),
    rgb: bool = typer.Option(False, '--rgb', help='转换为RGB格式'),
    quality: int = typer.Option(85, '--quality', '-q', help='图片质量，0-100，默认85'),
    output_type: _OutputType = typer.Option(_OutputType.normal, '--output-type', help='输出类型，0：普通输出，1：删除源文件，2：输出文件使用CRC32命名并删除源文件'),
):
    '图片格式转换'
    path = path or Path(os.getcwd())
    fileList: list[Path] = []
    if path.is_file():
        fileList.append(path)
    elif path.is_dir():
        extNameList = [x for x in src_format.strip().split('|')]
        fileList = [x for x in bpath.listFile(path, True) if x.suffix[1:].lower() in extNameList]
    if not fileList:
        return bcolor.printRed(f'未找到图片文件（{path}）')
    for file in fileList:
        with Image.open(file) as img:
            if rgb:
                img = img.convert('RGB')
            with bpath.useTempFile() as tempFile:
                img.save(tempFile, format=dst_format, quality=quality)
                outputFile = file.with_suffix(f'.{dst_format}')
                if output_type == _OutputType.crc_replace:
                    outputFile = outputFile.with_stem(await bfile.crc(tempFile))
                bpath.copy(tempFile, outputFile)
                if output_type in [_OutputType.replace_, _OutputType.crc_replace]:
                    if outputFile != file:
                        bpath.remove(file)
                bcolor.printGreen(f'{file} -> {outputFile}')


# ------------------------------------------------------------------------------------

@app.command()
@syncCall
async def tiny(
    path: Path = typer.Option(None, '--path', help='指定目录或具体图片文件，默认当前目录'),
    optimization: int = typer.Option(25, '--optimization', help='指定优化大小，如果没有达到优化效果就不处理，单位：%，默认25'),
    isKeepOriginal: bool = typer.Option(False, '--keep-original', help='保留原始图片'),
):

    keyList = [
        'MB3QmtvZ8HKRkXcDnxhWCNTXzvx6cNF3',
        '7L7X2CJ35GM1bChSHdT14yZPLx7FlpNk',
        'q8YLcvrXVW2NYcr5mMyzQhsSHF4j7gny',
    ]
    random.shuffle(keyList)

    btask.assertTrue(0 < optimization < 100, '优化大小必须在0-100之间')
    compression = 100 - optimization
    await block.setLimit(_TinyFile.runTiny, len(keyList))

    # 整理文件列表
    fileList = []
    path = path or Path(os.getcwd())
    if path.is_file():
        fileList.append(path)
    elif path.is_dir():
        fileList = [x for x in bpath.listFile(path, True) if x.suffix[1:].lower() in ['jpg', 'jpeg', 'png']]
    else:
        btask.abort('未找到图片文件', path)
    fileList.sort()

    # 将文件列表整理成 _TinyFile 对象
    fileList = [_TinyFile(x) for x in fileList]
    await asyncio.gather(*[x.updateInfo() for x in fileList])

    # 过滤掉已经处理过的图片
    for i in range(len(fileList)):
        file = fileList[i]
        if file.compression == 0:
            # 未处理过的图片，进行图片的压缩处理
            pass
        elif not file.isTiny and file.compression < compression:
            # 之前测试的压缩率不符合要求，不过现在符合了，进行图片的压缩处理
            pass
        else:
            # 要忽略掉的文件
            if file.isTiny:
                bcolor.printYellow(f'{file.file}（{file.compression - 100:.2f}% / 已压缩 / {file.getSizeDisplay()}）')
            else:
                bcolor.printMagenta(f'{file.file}（{file.compression - 100:.2f}% / 不处理 / {file.getSizeDisplay()}）')

            fileList[i] = Null
    fileList = [x for x in fileList if x]

    # 开始压缩
    taskList = [
        x.runTiny(keyList[i % len(keyList)], compression, isKeepOriginal)
        for i, x in enumerate(fileList)
    ]
    await asyncio.gather(*taskList)


# ------------------------------------------------------------------------------------


class _TinyFile:

    _endian: Final = '>'
    _sep = BytesWriter(_endian).writeStr('tiny').writeUint(9527).writeUint(709394).toBytes()

    @property
    def compression(self):
        return self._compression

    @property
    def isTiny(self):
        return self._isTiny

    @property
    def file(self):
        return self._file

    def __init__(self, file: XPath):
        self._file = file
        self._compression: float = 0.0
        self._isTiny: bool = False

    def getSizeDisplay(self):
        size = bpath.get(self._file).stat().st_size / 1024
        return f'{size:,.2f}KB'

    async def updateInfo(self):
        fileBytes = await bfile.readBytes(self._file)
        self._compression = 0.0
        self._isTiny = False
        blockAry = fileBytes.split(self._sep)
        if len(blockAry) > 1:
            info = BytesReader(self._endian, blockAry[1])
            size = info.readUint()
            if size == len(blockAry[0]):
                self._compression = round(info.readFloat(), 2)
                self._isTiny = info.readBool()

    async def _flushInfo(self, compression: float, isTiny: bool):
        self._compression = compression
        self._isTiny = isTiny
        content = await self._getPureContent()
        info = (
            BytesWriter(self._endian)
            .writeUint(len(content))
            .writeFloat(compression)
            .writeBool(isTiny)
            .toBytes()
        )
        content += self._sep + info
        await bfile.writeBytes(self._file, content)

    async def _getPureContent(self):
        content = await bfile.readBytes(self._file)
        content = content.split(self._sep)[0]
        return content

    @block.limit(1)
    async def runTiny(self, key: str, compression: float, isKeepOriginal: bool):
        content = await self._getPureContent()
        async with httpx.AsyncClient() as client:
            response = await client.post(
                'https://api.tinify.com/shrink',
                auth=('api', key),
                content=content,
                timeout=30,
            )
            response.raise_for_status()
            result = response.json()
            outputCompression = round(result['output']['ratio'] * 100, 2)
            if outputCompression < compression:
                # 下载文件
                url = result['output']['url']
                with bpath.useTempFile() as tempFile:
                    await bhttp.download(url, tempFile)
                    await _TinyFile(tempFile)._flushInfo(outputCompression, True)
                    outputFile = bpath.get(self._file)
                    if isKeepOriginal:
                        outputFile = outputFile.with_stem(f'{outputFile.stem}_tiny')
                    bpath.move(tempFile, outputFile, True)
                    bcolor.printGreen(f'{outputFile}（{outputCompression - 100:.2f}% / 压缩 / {self.getSizeDisplay()}）')
            else:
                # 不进行压缩
                await self._flushInfo(outputCompression, False)
                bcolor.printMagenta(f'{self._file} （{outputCompression - 100:.2f}% / 不处理 / {self.getSizeDisplay()}）')
